@@ -31,11 +31,11 @@
 #define __GTK_LABEL_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkmisc.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkmenu.h>
 
+
 G_BEGIN_DECLS
 
 #define GTK_TYPE_LABEL		  (gtk_label_get_type ())
@@ -56,30 +56,31 @@ struct _GtkLabel
   GtkMisc misc;
 
   /*< private >*/
-  gchar  *label;
-  guint   jtype            : 2;
-  guint   wrap             : 1;
-  guint   use_underline    : 1;
-  guint   use_markup       : 1;
-  guint   ellipsize        : 3;
-  guint   single_line_mode : 1;
-  guint   have_transform   : 1;
-  guint   in_click         : 1;
-  guint   wrap_mode        : 3;
-  guint   pattern_set      : 1;
+  gchar  *GSEAL (label);
+  guint   GSEAL (jtype)            : 2;
+  guint   GSEAL (wrap)             : 1;
+  guint   GSEAL (use_underline)    : 1;
+  guint   GSEAL (use_markup)       : 1;
+  guint   GSEAL (ellipsize)        : 3;
+  guint   GSEAL (single_line_mode) : 1;
+  guint   GSEAL (have_transform)   : 1;
+  guint   GSEAL (in_click)         : 1;
+  guint   GSEAL (wrap_mode)        : 3;
+  guint   GSEAL (pattern_set)      : 1;
+  guint   GSEAL (track_links)      : 1;
 
-  guint   mnemonic_keyval;
+  guint   GSEAL (mnemonic_keyval);
 
-  gchar  *text;
-  PangoAttrList *attrs;
-  PangoAttrList *effective_attrs;
+  gchar  *GSEAL (text);
+  PangoAttrList *GSEAL (attrs);
+  PangoAttrList *GSEAL (effective_attrs);
 
-  PangoLayout *layout;
+  PangoLayout *GSEAL (layout);
 
-  GtkWidget *mnemonic_widget;
-  GtkWindow *mnemonic_window;
+  GtkWidget *GSEAL (mnemonic_widget);
+  GtkWindow *GSEAL (mnemonic_window);
 
-  GtkLabelSelectionInfo *select_info;
+  GtkLabelSelectionInfo *GSEAL (select_info);
 };
 
 struct _GtkLabelClass
@@ -96,11 +97,13 @@ struct _GtkLabelClass
   void (* populate_popup)   (GtkLabel       *label,
                              GtkMenu        *menu);
 
+  gboolean (*activate_link) (GtkLabel       *label,
+                             const gchar    *uri);
+
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
   void (*_gtk_reserved3) (void);
-  void (*_gtk_reserved4) (void);
 };
 
 GType                 gtk_label_get_type          (void) G_GNUC_CONST;
@@ -174,6 +177,11 @@ void         gtk_label_set_single_line_mode  (GtkLabel *label,
                                               gboolean single_line_mode);
 gboolean     gtk_label_get_single_line_mode  (GtkLabel *label);
 
+G_CONST_RETURN gchar *gtk_label_get_current_uri (GtkLabel *label);
+void         gtk_label_set_track_visited_links  (GtkLabel *label,
+                                                 gboolean  track_links);
+gboolean     gtk_label_get_track_visited_links  (GtkLabel *label);
+
 #ifndef GTK_DISABLE_DEPRECATED
 
 #define  gtk_label_set           gtk_label_set_text
