@@ -70,9 +70,17 @@ _gtk_socket_windowing_realize_window (GtkSocket *socket)
 			GDK_WINDOW_XWINDOW (window),
 			&xattrs);
 
+  /* Sooooo, it turns out that mozilla, as per the gtk2xt code selects
+     for input on the socket with a mask of 0x0fffff (for god knows why)
+     which includes ButtonPressMask causing a BadAccess if someone else
+     also selects for this. As per the client-side windows merge we always
+     normally selects for button press so we can emulate it on client
+     side children that selects for button press. However, we don't need
+     this for GtkSocket, so we unselect it here, fixing the crashes in
+     firefox. */
   XSelectInput (GDK_WINDOW_XDISPLAY (window),
 		GDK_WINDOW_XWINDOW (window), 
-		xattrs.your_event_mask | 
+		(xattrs.your_event_mask & ~ButtonPressMask) |
 		SubstructureNotifyMask | SubstructureRedirectMask);
 }
 
@@ -600,7 +608,7 @@ _gtk_socket_windowing_filter_func (GdkXEvent *gdk_xevent,
       {
 	XReparentEvent *xre = &xevent->xreparent;
 
-	GTK_NOTE (PLUGSOCKET, g_message ("GtkPlug: ReparentNotify received\n"));
+	GTK_NOTE (PLUGSOCKET, g_message ("GtkSocket - ReparentNotify received"));
 	if (!socket->plug_window && xre->parent == GDK_WINDOW_XWINDOW (widget->window))
 	  {
 	    _gtk_socket_add_window (socket, xre->window, FALSE);
